#! /usr/bin/python2.7
# -*- coding: utf-8 -*- 

###############################
#  @author: Jonathan Verneau  #
#	 PhD Student, URMITE  # 
###############################

'''
NAME
  %(progname)s

VERSION
  %(version)s

CATEGORY
  Metagenomes; Bioinformatics

AUTHOR
  Jonathan VERNEAU <jonathan8313@gmail.com>

DESCRIPTION
  A pipeline to search interest sequences in metagenomes
'''

# Library of python
from Bio import SeqIO
import argparse
import os
import time
import datetime
import sys
import fnmatch
import glob
import operator
import csv
import shutil
import smtplib
import mimetypes
import tarfile
import zipfile
import ftplib
import platform
import inspect
from Tkinter import * 
from tkMessageBox import *
from tkFileDialog import *
from ScrolledText import *
from email.mime.application import MIMEApplication
from email.MIMEMultipart import MIMEMultipart
from email.MIMEText import MIMEText
from config import *

######################## Functions GUI ################################################################################################
def about():
	showinfo("About", "MG-Digger\nA pipeline to search interest sequences in metagenome\n\nContact: Jonathan Verneau \n PhD URMITE\n<jonathan8313@gmail.com>")

def update():
	'''Update NCBI databases'''
	def extract_file(path, to_directory='.'):
	    if path.endswith('.zip'):
		opener, mode = zipfile.ZipFile, 'r'
	    elif path.endswith('.tar.gz') or path.endswith('.tgz'):
		opener, mode = tarfile.open, 'r:gz'
	    elif path.endswith('.tar.bz2') or path.endswith('.tbz'):
		opener, mode = tarfile.open, 'r:bz2'
	    else: 
		raise ValueError, "Could not extract `%s` as no appropriate extractor is found" % path
	    
	    cwd = os.getcwd()
	    os.chdir(to_directory)
		    
	    try:
		file = opener(path, mode)
		try: file.extractall()
		finally: file.close()
	    finally:
		os.chdir(cwd)

	def dowload():
		url = line.strip()
		filename = url.split('/')[-1]
		print "Download %s as %s" % (url, filename)

	
	if askokcancel('Information', "Update database takes over 100 GB ..."):
		start_Time = time.time()
		dir = os.getcwd()
		#************************************************ Blast database update*****************************************************************************************
	
		# If folder doesn't exist , create it!
		if not os.path.exists(Blast_Folder):
			os.makedirs(Blast_Folder)

		# Creation of files
		lf=[Megavirales_list,Viruses_list,Eukaryota_list,Bacteria_list,Archeae_list]
		for i in lf:
			if not os.path.isfile(i):
				f = open(i,"w")
				f.write('# Add line per line a list of GIs nucleotide and protein of species you want studies')
				f.close()
		
		# Move to Blast folder
		os.chdir(Blast_Folder)
		
		
		#Login NCBI FTP
		ftp = ftplib.FTP("ftp.ncbi.nih.gov")
		ftp.login('anonymous','Anonymous') 

	
		directory ="/blast/db" #dir i want to download files from, can be changed or left for user input
		ftp.cwd(directory)

		# Display on the screen	
		print "5%"
		print "# Download NR Database "
		
		for filename in ftp.nlst('nr*'): 
			if filename not in ftp.nlst('*.md5'): 
				fhandle = open(filename, 'wb')
				print 'Downloading ' +filename +' ...' 
				ftp.retrbinary('RETR '+filename, fhandle.write)
				fhandle.close()	
				print 'Extracting ' +filename +' ...'				
				extract_file(filename)
				os.remove(filename)


		# Display on the screen	
		print "25%"
		print "# Download NT Database "
		for filename in ftp.nlst('nt*'): 
			if filename not in ftp.nlst('*.md5'): 
		  		fhandle = open(filename, 'wb')
				print 'Downloading '+filename+' ...' 
		    		ftp.retrbinary('RETR '+filename, fhandle.write)
		   		fhandle.close()	
				extract_file(filename)
				os.remove(filename)
		
		# Display on the screen	
		print "45%"
		print "# Download 16S Database "
		# Download files
		print 'Downloading 16SMicrobial.tar.gz ...'
		ftp.retrbinary("RETR 16SMicrobial.tar.gz" ,open("16SMicrobial.tar.gz", 'wb').write)
		extract_file('16SMicrobial.tar.gz')
		os.remove('16SMicrobial.tar.gz')
		os.chdir(dir)
		
		#************************************************ Taxonomy database update *****************************************************************************************

		# If folder doesn't exist , create it!
		if os.path.exists(TaxID_Folder):
			shutil.rmtree(TaxID_Folder)
			os.makedirs(TaxID_Folder) 
		else:
			os.makedirs(TaxID_Folder)

		# Move to the taxonomy database folder
		os.chdir(TaxID_Folder)

		#Login NCBI FTP
		ftp = ftplib.FTP("ftp.ncbi.nih.gov")
		ftp.login('anonymous','Anonymous') 


		# Display on the screen
		print "80"
		print "# Updating of taxonomy informations "

		# Download taxdump
		print 'Downloading taxdump.tar.gz ...'
		ftp.retrbinary("RETR pub/taxonomy/taxdump.tar.gz" ,open("taxdump.tar.gz", 'wb').write)
		extract_file("taxdump.tar.gz")
		os.remove("taxdump.tar.gz")
	 	os.remove("citations.dmp")
		os.remove("division.dmp")
		os.remove("gencode.dmp")
		os.remove("merged.dmp")
		os.remove("delnodes.dmp")
		os.remove("gc.prt") 
		os.remove("readme.txt")

		os.chdir(dir)
		print "\n# Update NCBI database done in "+str(endTime(start_Time))		
		showinfo('Information', 'Update databases done')
	else:
		showwarning('Information', 'Update canceled!')

	
def select_metagenome():
	''' Select metagenome '''
	fic=""
	src=askopenfilename(title ="Open file : " , parent=root, initialfile =fic , filetypes = [("Fasta file",("*.fasta","*.faa","*.fna","*.fa","*.fst","*.fa","*.txt")), ("All",".*")])
	path=os.path.abspath(src)
	text_Metagenome.set(path)
	
def select_item():
	''' Select item '''
	fic=""
	src=askopenfilename(title ="Open file : " , \
		     parent=root, initialfile =fic ,\
		     filetypes = [("Fasta file",("*.fasta","*.faa","*.fna","*.fa","*.fst","*.fa","*.txt")), ("All",".*")])
	path=os.path.abspath(src)
	text_Item.set(path)


def run():
	''' Lauch the script '''
	group = listbox1.get(listbox1.curselection())
	evalue2 = evalue.get()
	cpu  = cpus.get()
	mail2 = mail.get()
	type = typeblast.get()

	os.environ["BLASTDB"] = Blast_Folder

	if type =='16S':
		if group in g2 :
			main_16S(text_Metagenome,group,evalue2,cpu,mail2)
		else:
			showinfo('Information', 'Error on arguments')
	else :
		if type in p :
			main(type,text_Metagenome,text_Item,group,evalue2,cpu,mail2)
		else :
			showinfo('Information', 'Error on arguments')


def setpath():
	''' Set parameters '''
	def save_command():
		file = open(folder_s+'/config.py','w')
		data = textPad.get('1.0', END+'-1c')
		file.write(data)
		file.close()

	def close_command():
		data = textPad.get('1.0', END+'-1c')
		contents =open(folder_s+'/config.py','rb').read()
		if contents != data:
			if askyesno("Quit editor", "Do you want to save changes in configuration file?"):
				file = open(folder_s+'/config.py','w')
				file.write(data)
				file.close()
		app.destroy()
	

	app = Tk(className=" Editor")

	menubar = Frame(app, borderwidth=0)
	menubar.pack(fill="both", expand="yes")
	Button(menubar, text ="Save", command=save_command).pack(side=LEFT, padx=5, pady=5)
	Button(menubar, text ="Close", command=close_command).pack(side=LEFT, padx=5, pady=5)

	textPad = ScrolledText(app, width=100, height=80)
	file = open(folder_s+'/config.py','rb')
	contents = file.read()
	textPad.insert('1.0',contents)
	file.close()

	textPad.pack()
	app.protocol("WM_DELETE_WINDOW", close_command)
	app.mainloop()
	

def cleaner(): 
	def extension_test_cleaner(file):
		l = ['.fasta','.fastq','.fa','.fas','.fna','.ffn','.faa','.frn','.txt','.sra']	
		ext = os.path.splitext(file)[-1].lower()
		if ext not in l:
			showinfo('Information', file+" has not good extension!")
			sys.exit(0)
			

	def fastq_Conversion(file):
		print "# Please wait, conversion fastq to fasta in progress..."
		dir = os.getcwd()
		file_Name = os.path.basename(file)
		prefix = os.path.splitext(file_Name)[0]
		fasta_file = dir+"/"+prefix+".fasta"
		SeqIO.convert(file,'fastq',fasta_file,'fasta')
		return fasta_file

	def sizeof_fmt(num):
		'''Size of metagenome by  Fred Cirera '''
		for x in ['bytes','KB','MB','GB','TB']:
			if num < 1024.0:
				return "%3.1f%s" % (num, x)
			num /= 1024.0

	def select_metagenome_cleaner():
		''' Select metagenome for cleaner '''
		fic=""
		src=askopenfilename(title ="Open file : " , parent=root, initialfile =fic ,filetypes = [("Fasta file",("*.fasta","*.faa","*.fna","*.fa","*.fst","*.fa","*.txt","*.sra","*.fastq")), ("All",".*")])
		path=os.path.abspath(src)
		text_Metagenome_cleaner.set(path)

	def main_cleaner(metagenome,minus,n):
		'''Remove duplicate sequences, remove too short sequences ( the user defines the minimum length) and remove sequences which have too many unknown nucleotides (N) ( the user defines the % of N is allows ) '''
	
		
		start_Time = time.time()
		metagenome = os.path.abspath(metagenome)	
	
		extension_test_cleaner(metagenome)
	
		extension = os.path.splitext(metagenome)[-1].lower()
		metagenome_Name = os.path.basename(metagenome)	
		print "# Start cleaning metagenome "+metagenome_Name+" ..."
		
		# Fastq to fasta
		if extension == '.fastq':
			print "# Start conversion of "+metagenome_Name
			print "# Please wait, conversion fastq to fasta in progress..."
			prefix = os.path.splitext(metagenome_Name)[0]
			metagenome_tmp = fastq_Conversion(metagenome)
			os.remove(metagenome)
			metagenome = metagenome_tmp

		# SRA to fasta
		if extension == ".sra":
			print "# Start conversion of "+metagenome_Name
			print "# Please wait, conversion sra to fasta in progress..."
			#SRA Toolkit
			path_metagenome = str(metagenome.split(metagenome_Name)[0])	
			
			if platform.system() == 'Linux':
				os.system(sra_prog_folder+"_linux/fastq-dump --fasta "+metagenome+" -O "+path_metagenome)
			if platform.system() == 'Windows':
				os.system(sra_prog_folder+"_windows/fastq-dump --fasta "+metagenome+" -O "+path_metagenome)
			if platform.system() == 'Darwin':
				os.system(sra_prog_folder+"_mac/fastq-dump --fasta "+metagenome+" -O "+path_metagenome)
				
			metagenome_Name = str(metagenome_Name.split('.sra')[0]+".fasta")
			metagenome = path_metagenome + metagenome_Name
	
	
	
	
		metagenome_Name = os.path.basename(metagenome)	
		print "# Start cleaning metagenome "+metagenome_Name+" ..."

		# Create our hash table to add the sequences
		dic_sequences={}
		# Create counters
		cpt1=0
		cpt2=0



		# Lenght of metagenome before cleaning
		taille_mg= sizeof_fmt(os.path.getsize(metagenome))
			

		# Using the biopython fasta parse we can read our fasta input
		for sequence in SeqIO.parse(metagenome, "fasta"):
			# Count the number of reads in metagenome 			
			cpt1= cpt1+1	
		
			if sequence.seq.tostring() != '':
	
				# Take the current sequence
				sequence_fasta=str(sequence.seq).upper()
			
				# Delete all end N
				if (sequence_fasta[len(sequence_fasta)-1] == 'N'):
					sequence_fasta = sequence_fasta.rstrip('N')
				

				# Check if the current sequence is according to the user parameters
				if (len(sequence_fasta)>=minus and (float(sequence_fasta.count("N"))/float(len(sequence_fasta)))*100<=n):
					# If the sequence passed in the test "is It clean?" and It isnt in the hash table , the sequence and Its id are going to be in the hash          		
					if sequence_fasta not in dic_sequences:
						sequence_description = str(sequence.description).replace('\t','_').replace(' ','_')
						dic_sequences[sequence_fasta]=sequence_description
						cpt2=cpt2+1

		## Write the clean sequences
		# Create a file in the results directory where you ran this script
		output_file=open(metagenome,"w")

		#Just Read the Hash Table and write on the file as a fasta format
		for sequence in dic_sequences:
			output_file.write(">"+dic_sequences[sequence]+"\n"+sequence+"\n")
		output_file.close()

		del dic_sequences
		# Lenght of metagenome after cleaning
		taille_mg2 = sizeof_fmt(os.path.getsize(metagenome))
	

		# Display on the screen
		m = str(cpt1)+" sequences before cleaning\n"+str(cpt2)+" sequences after cleaning\n"+str(cpt1-cpt2)+" drop sequences\n# "+metagenome_Name+" cleaned in "+str(endTime(start_Time_cleaner))

		showinfo('MG-Digger information', m)
		
		sys.exit(0)

	def run_cleaner():
		minus_get = int(minus.get())
		n_get = int(n.get())
		if minus_get > 0 and n_get < 100  :
			main_cleaner(text_Metagenome_cleaner.get(),minus_get,n_get)
		else :
			showwarning('MG-Digger warnings', 'Error in the parameters')

		
	## Construction of window tool
	app = Toplevel()
	app.resizable(0,0)


	## Menu bar
	menubar2 = Menu(app)

	menu1 = Menu(menubar2, tearoff=0)
	menu1.add_command(label="Exit", command=app.quit)
	menubar2.add_cascade(label="File", menu=menu1)

	menu2 = Menu(menubar, tearoff=0)
	menu2.add_command(label="About...", command=about)
	menubar2.add_cascade(label="Help", menu=menu2)

	app.config(menu=menubar2)

	try:
		# Logo
		canvas = Canvas(app,width=100, height=100)
		canvas.create_image(0, 0, anchor=NW, image=photo)
		canvas.pack()
	except:
		pass
	
	# Tool title
	label_title = Label(app, text="MG-Digger cleaner v1.0",bg="green").pack()


	# Choose files
	label_files = LabelFrame(app, text="Choose the files:", borderwidth=5 ,padx=20, pady=20)
	label_files.pack(fill="both", expand="yes")

	# Choose metagenome
	label_metagenome = Frame(label_files, borderwidth=0)
	label_metagenome.pack(fill="both", expand="yes")

	Button(label_metagenome, text ="Select Metagenome", command=select_metagenome_cleaner).pack(side=LEFT, padx=5, pady=5)
	text_Metagenome_cleaner = StringVar() 
	LabelResultat = Label(label_metagenome, textvariable = text_Metagenome_cleaner, fg ='red', bg ='white')
	LabelResultat.pack(side = LEFT, padx = 5, pady = 5)


	# Option window.
	label_option = LabelFrame(app, text="Option:", borderwidth=5, padx=20, pady=20)
	label_option.pack(fill="both", expand="yes")




	# lenght min
	label_lenght = LabelFrame(label_option, text="Enter the minimum lenght of reads", borderwidth=2,relief=GROOVE)
	label_lenght.pack(side=TOP)

	minus = StringVar() 
	minus.set("40")
	entree = Entry(label_lenght, textvariable=minus, width=30, highlightthickness= 40)
	entree.pack()

	# Percent of N
	label_n = LabelFrame(label_option, text="Enter the percent maximum of nucleotide N per reads", borderwidth=2, relief=GROOVE)
	label_n.pack()

	n = StringVar()
	n.set("20")
	entree = Entry(label_n, textvariable=n, width=30, highlightthickness= 40)
	entree.pack()

	# Buttons
	Button(app, text ='Run', command=run_cleaner).pack(side=LEFT, padx=5, pady=5)
	Button(app, text ='Cancel', command=app.quit).pack(side=RIGHT, padx=5, pady=5)

	




######################## Functions ################################################################################################

def display_time(secondes): 
	''' Convert time for human reading '''
 	s = int(secondes)
	conv = (31536000,2592000,86400,3600,60,1) 
	result = [0,0,0,0,0,0]
	date = ['years','months','days','hours','minutes','seconds']
	i=0 
	resultf=''
	 
	while s>0: 
		result[i], s = divmod(s,conv[i] )
		i+=1 

	for j in range(len(result)):
		if result[j] != 0:
			resultf =resultf+str(result[j])+" "+str(date[j])+" "

	return resultf.rstrip(" ")


def endTime(start):
	''' Calculate the duration of analysis'''
	end_Time = time.time()
	return display_time(end_Time - start)



def extension_test(file):
	''' Test if file is correct format '''
	l = ['.fasta','.fa','.fas','.fna','.ffn','.faa','.frn','.txt']	
	ext = os.path.splitext(file)[-1].lower()
	if ext not in l:

		showinfo('Information', file+' has not good extension!')
		sys.exit(0)


def metagenome_sequences_match(metagenome_file,csv_file,blast):
	''' Extract reads of metagenome in a fasta file'''
	print "Sequences extraction start...." 
	
	# Open the CSV file from blast program analyze.
	with open(csv_file,"r") as file:
		lines=file.readlines()
		
	# Create dictionnary with all reads.
	d={}
	
	# Add reads in dictionnary
	if blast == 'tblastn':
		for line in lines:
			d[line.split("\t")[1]]=1
	else :
		for line in lines:
			d[line.split("\t")[0]]=1	
	
		
	cpt=0
	
	# Arguments name 
	csv_name = os.path.basename(csv_file)
	with open("seqmatch_for_"+csv_name+"_analyse.fasta","w") as f:
		# Using the biopython fasta parse we can read our fasta input
		for seq_record in SeqIO.parse(metagenome_file, "fasta"):
			# Count the reads total of metagenome
			cpt += 1
			# Extract the reads in metagenome_file present in dictionnary
			if seq_record.description in d:
				f.write(">"+str(seq_record.description)+"\n"+str(seq_record.seq)+"\n")
				# Delete the item in dictionnary			
				d.pop(seq_record.description,None)
	
	del d
	
	print "End of extraction!"
	
	return cpt
	
def extract_per_items(metagenome_file,fasta_file,csv_file):
	''' Extract reads of result csv file in fasta files with species'''
	m=os.path.basename(metagenome_file)	
	d={}	
	s=''
	
	with open(csv_file,"r") as f1:
		lines=f1.readlines()

					
	for seq_record in SeqIO.parse(fasta_file, "fasta"):
		# Take the current sequence		
		d[seq_record.description]= seq_record.seq
	
	with open("final_analysis_reads_file_for_"+m, "w") as f2:
		for line in lines:
			l=line.split('\t')
			item=str(l[4].split(";")[0])
			if l[0] in d:
				f2.write(">"+str(l[0])+"\t["+item+"]\n"+str(d[l[0]])+"\n")
				d.pop(l[0],None)

			
def one_hsp(t,csv_file,g):
	''' Extract one hsp per read in a csv file'''
	# Open the CSV file from blast program analyze.
	with open(csv_file,"r") as file:
		lines=file.readlines()
	
	d={}
	
	for line in lines:
		d[line.split("\t")[0]]=1
		
	with open("result_one_hsp_per_read.csv","w") as f:
		if g != 'All':
			listv=''
			if g == 'Megavirales' and os.path.isfile(Megavirales_list) and os.path.getsize(Megavirales_list) > 0 :
				try:
					with open(Megavirales_list,"r") as file2:
						listv=file2.readlines()
				except IOError:
					print "Error on "+g+" GIs list! Analysis of all species ..."

			elif g == 'Viruses' and os.path.isfile(Viruses_list) and os.path.getsize(Viruses_list) > 0:
				try:
					with open(Viruses_list,"r") as file2:
						listv=file2.readlines()
				except IOError:
					print "Error on "+g+" GIs list! Analysis of all species ..."

			elif g == 'Eukaryota' and os.path.isfile(Eukaryota_list) and os.path.getsize(Eukaryota_list) > 0 :
				try:
					with open(Eukaryota_list,"r") as file2:
						listv=file2.readlines()
				except IOError:
					print "Error on "+g+" GIs list! Analysis of all species ..."

			elif g == 'Bacteria' and os.path.isfile(Bacteria_list) and os.path.getsize(Bacteria_list) > 0 :
				try:
					with open(Bacteria_list,"r") as file2:
						listv=file2.readlines()
				except IOError:
					print "Error on "+g+" GIs list! Analysis of all species ..."

			elif g == 'Archeae' and os.path.isfile(Archeae_list) and os.path.getsize(Archeae_list) > 0 :
				try:
					with open(Archeae_list,"r") as file2:
						listv=file2.readlines()
				except IOError:
					print "Error on "+g+" GIs list! Analysis of all species ..."

		
			if listv != '# Add line per line a list of GIs nucleotide and protein of species you want studies':

				list=[]
		
				for i in listv:
					list.append(i.rstrip("\n"))


				for line in lines:
					l=line.split("\t")

					if l[0] in d:
						gi = str(l[1].split("|")[1])
						if gi in list:
							f.write(line)		
						d.pop(l[0],None)
						
			else :
				if g == 'Megavirales':
					g = 'Viruses'
				for line in lines:
					if line[0] != '#':
						l=line.split("\t")
						if l[3] == g:
							if l[0] in d:
								f.write(line)	
								d.pop(str(l[0]),None)				 

	
		else:	
			for line in lines:
				l=line.split("\t")
				if l[0] in d :
						f.write(line)
						d.pop(str(l[0]),None) 
	
	del d
	




def get_stats_species(resultonehsp_file):
	''' get the number per species '''
	# Sort colomn scientific names				
	sort_csv(resultonehsp_file,5)

	# Creation of statistic file
	with open(resultonehsp_file, "r") as file:
		lines = file.readlines()
								
	d={}
	for l in lines:
		item = l.split('\t')[4].split(';')[0]
		d[item]=0
	for l in lines:
		item = l.split('\t')[4].split(';')[0]
		if item in d:
			d[item] = d[item]+1
					

	with open("stats_tmp.csv", "w") as fw:
		for i in d:
			fw.write(str(d[i])+"\t"+i+"\n")

	del d
							
					

def taxonomy(tmpfile,finalfile):
	'''have taxonomy studies per species for statistic final file'''
	with open(nodes_file, "r") as file:
		lines=file.readlines()
		d={}
		for line in lines:
			parent = int(line.split("\t")[2])
			taxid = int(line.split("\t")[0])
			d[taxid] = parent

	with open(names_file, "r") as file:
		lines2=file.readlines()
		d2={}
		liste=[]
		l=[]

		for i in range(10000000):
			liste.extend("0")

		for line2 in lines2:
			taxid2 = int (line2.split("\t")[0])
			names = line2.split("\t")[2]
			if line2.split("\t")[6] == "scientific name":
					d2[taxid2] = names
					liste[taxid2] = names



	with open(tmpfile, "r") as file:
		lines3=file.readlines()
		for line3 in lines3:
			l.append(line3.strip(' \n\t\r'))


	with open(finalfile, "w") as file:

		for i in l:
			if i.split("\t")[1] == 'N/A':
				file.write(str(i)+"\tN/A\tN/A\n")
			else:
				try:
					if i.split("\t")[1] in liste:					
						j = liste.index(i.split("\t")[1])				
						taxid_s = liste[j] 
						parent_s = d[j]
						lineage=''
						while True :
							lineage = d2[parent_s]+";"+lineage
							taxid_s = parent_s
							parent_s = d[taxid_s]
							if parent_s == 1:
								break
						file.write(str(i)+"\t"+str(j)+"\t"+lineage+''+str(liste[j])+"\n")
				
					else:
						for line in open(names_file):
								if i.split("\t")[1] in line:
									taxid_s = int(line.split("\t")[0])
									parent_s = d[taxid_s]
									lineage=''

									while True :
										lineage = d2[parent_s]+";"+lineage
										taxid_s = parent_s
										parent_s = d[taxid_s]
										if parent_s == 1:
											break

									file.write(str(i)+"\t"+str(line.split("\t")[0])+"\t"+lineage+''+str(i.split("\t")[1])+"\n")
									break
							
				except:
					file.write(str(i)+"\tN/A\tN/A\n")
	del d
	del d2

							
def del_folder(folder):
	''' Functiun to delete empty files in folder'''
	files = glob.glob(folder+"/*")
	for f in files:
		if os.path.getsize(f) == 0 : 
			os.remove(f)

def send_mail(address,title,msgToSend,filename=''):
    	''' Functiun to send mail in the end of analysis'''
	try:
		msg = MIMEMultipart()
		msg['From'] = 'Not_respond_MG-Digger_tools@gmail.com'
		msg['To'] = str(address)
		msg['Subject'] = str(title)
		message = str(msgToSend)+'\n\n\nThanks you for using MG-Digger'
		msg.attach(MIMEText(message))
	
		if filename != '':
			with open(filename,'rb') as file:
				att = MIMEApplication(file.read(),_subtype="png")
				att.add_header('Content-Disposition','attachment',filename=filename)
				msg.attach(att)
	
		mailserver = smtplib.SMTP('smtp.gmail.com', 587)
		mailserver.ehlo()
		mailserver.starttls()
		mailserver.ehlo()
		mailserver.login(sender_mail, password_mail)
		mailserver.sendmail(sender_mail, address, msg.as_string())
		mailserver.quit()
	except:
		print "Error when sending mail!"

def sort_csv(file_csv, column):
	''' Sort csv file per column'''
	c = int(column)-1
	data= csv.reader(open(file_csv),delimiter='\t')
	sortedlist = sorted(data, key=operator.itemgetter(c))
	with open(file_csv, "w") as file:
		for i in sortedlist:
			for j in i:
				file.write(str(j)+"\t")
			file.write("\n")

def estimate_time(t,cpu_comp,sequences):
	''' Estimate the duration of time of blast analysis'''
	
	if t == 'blastx':
		if sequences > cpu_comp: 	
			time_analysis=sequences*400/cpu_comp
		else: 
			time_analysis=400
		time_analysis_converted = time.time() + time_analysis

	else: 
		if sequences > cpu_comp: 	
			time_analysis=sequences*50/cpu_comp
		else: 
			time_analysis=50
		time_analysis_converted = time.time() + time_analysis
	
	return datetime.datetime.fromtimestamp(time_analysis_converted).strftime('%Y-%m-%d %H:%M:%S')
					

def main_16S(metagenome,group,evalue,cpus,mail):
	''' Main Programm for 16 Microbial analysis'''	
	start_Time = time.time()
	extension_test(metagenome)
	metagenome = os.path.abspath(metagenome)
	
	name_Metagenome = os.path.basename(metagenome)

	prefix_Metagenome = os.path.splitext(name_Metagenome)[0]
	analysis_Folder = prefix_Metagenome+"_vs_16S_Microbial"

	if os.path.exists(analysis_Folder):
		if askyesno('Information', "Folder exist!\nDo you want to delete it?"):
			shutil.rmtree(analysis_Folder)
			os.mkdir(analysis_Folder) 
		else:
			showwarning('Information', 'Update canceled!')

	else: 
		os.mkdir(analysis_Folder) 


	print "Start Analysis "+name_Metagenome+" and 16S..."

	# Move to metagenome/subject folder
	work_Folder=os.getcwd()
	directory=work_Folder+"/"+analysis_Folder
	os.chdir(directory)

	# Count of number read of metagenome
	cptreads = 0
	for sequences in SeqIO.parse(metagenome, "fasta"):
		cptreads += 1

	## Make a blastn for nucleotidic Orfs subject.	

	print "10%"
	print "# blastn "+name_Metagenome+" vs 16S..."
	print "# Analysis of "+str(cptreads)+" reads..."
	print "# Estimation of finished time: "+str(estimate_time("16S",cpus, cptreads))
	# Creation of a cvs file with the tblastn program.
	seqdumpfile=prefix_Metagenome+"_VS_16S_blastn.csv"


	# Run of blastn program.
	os.system ("blastn -query "+metagenome+" -db 16SMicrobial -perc_identity 97 -qcov_hsp_perc 75 -max_hsps 1 -num_threads "+str(cpus)+" -evalue "+str(evalue)+" -max_target_seqs 10 -outfmt \"6 qseqid sseqid stitle sskingdoms sscinames bitscore score qcovs evalue pident length qlen slen qstart qend sstart send qseq sseq\" -out "+seqdumpfile)

	# Dispaly on the screen.		
	print "75%"
	print "# Sort of reads to analysis"				

	if os.path.getsize(seqdumpfile) > 0 and os.path.isfile(seqdumpfile):  
			
		print "# Extraction of one hsp per read..."
			
		# Extract one hsp per reads.
		one_hsp(typeblast,seqdumpfile,group)
	
		resultonehsp="result_one_hsp_per_read.csv"
		if  os.path.getsize(resultonehsp) > 0 and os.path.isfile(resultonehsp):			
			print "# Creation of final reads fasta file..."
				
			# Extract reads in final fasta file.				
			extract_per_items(metagenome,metagenome,resultonehsp)
			
			# Count the numbers of final reads.
			cpt2=0
			with open(resultonehsp) as f:
				cpt2=sum(1 for _ in f)
					
			print "85%"
			print "# Get lineage for taxonomy studies for "+str(cpt2)+" species..."
					
			# Get  stats file
			get_stats_species(resultonehsp)
					
			# Name of statistic files.
			numberitemstmp="stats_tmp.csv"
			statsfinal="Stats_"+name_Metagenome+"_VS_16S_Microbial.csv"

			# Get lineage of species
			taxonomy(numberitemstmp,statsfinal)
				
			# Sort colomn lineage				
			sort_csv(statsfinal,4)

			# Add explicit line in read files
			with open(statsfinal, 'r+') as file:
				text = file.read()
	   			file.seek(0)
				file.write("# Metagenome: "+name_Metagenome+";\n# Subjects: 16S Microbial;\n# E-value: "+str(evalue)+";\n# Reads total: "+str(cptreads)+";\n# Reads analyzed: "+str(cpt2)+";\n# Analysis time: "+str(endTime(start_Time))+";\n# Number of CPUs used: "+str(cpus)+";\n#\n# 16S reads\n# Numbers\tSubject name\tTaxonomy ID\tLineage\n"+text)
			with open(resultonehsp, 'r+') as file:
				text = file.read()
	       			file.seek(0)
				file.write("Query Seq-id\tSubject Seq-id\tTitle\tGroup\tScientific name(s)\tMax score\tTotal score\tQuery coverage\tE-value\tPercentage of identical matches\tAlignment length (nucleotide)\tQuery sequence length (nucleotide)\tSubject sequence length (nucleotide)\tStart of alignment in query (nucleotide)\tEnd of alignment in query (nucleotide)\tStart of alignment in subject (nucleotide)\tEnd of alignment in subject (nucleotide)\tAligned part of query sequence\tAligned part of subject sequence\n"+text)
				

			# Display on the screen
			print "100%"
			print "# Analysis metagenome "+name_Metagenome+" with 16S Microbial end!"
			print str(cpt2)+" reads analyzed in "+str(endTime(start_Time))
				
			# Remove temporary files
			os.remove(numberitemstmp)
	
			# Send mail
			if sys.argv[6] != '':
				t='Analysis of '+name_Metagenome+' VS 16S Microbial finished'
				m='Analysis of '+name_Metagenome+' VS 16S Microbial finished in '+str(endTime(start_Time))+'\n'+str(cpt2)+' reads were analyzed.\nPlease verify the folder : '+directory
				send_mail(mail,t,m,statsfinal)
							
		
		else :
			#Delete empty files
			del_folder(directory)

			# Add satistic files	
			with open("Stats_"+name_Metagenome+"_VS_16S_Microbial.txt", "w") as file:
				file.write("# Metagenome: "+name_Metagenome+";\n# Subjects: 16S;\n# E-value: "+str(evalue)+";\n# Reads total: "+str(cptreads)+";\n# Analysis time: "+str(endTime(start_Time))+";\n#\n#\nNo match between "+name_Metagenome+" and 16S !")

			# Display on the screen	
			print "100%"
			print "# No 16S in "+prefix_Metagenome+"!"
			print "Analysis finished in "+str(endTime(start_Time))
			
			# Send mail
			if sys.argv[6] != '':
				t='Analysis of '+name_Metagenome+' VS 16S Microbial finished'
				m='Analysis of '+metagenome+' VS 16S Microbial finished in '+str(endTime(start_Time))+'\nNo 16S reads.'
				send_mail(mail,t,m)
					

		
	else :
		#Delete empty files
		del_folder(directory)
			
		# Add satistic files	
		with open("Stats_"+name_Metagenome+"_VS_16SMicrobial.txt", "w") as file:
			file.write("# Metagenome: "+name_Metagenome+";\n# Subjects: 16S Microbial;\n# E-value: "+str(evalue)+";\n# Reads total: "+str(cptreads)+";\n# Analysis time: "+str(endTime(start_Time))+";\n#\n#\nNo match between "+name_Metagenome+" and 16S !")
		# Display on the screen	
		print "100%"
		print "# No 16S in "+name_Metagenome+"!"
		print "Analysis finished in "+str(endTime(start_Time))

		# Send mail
		if sys.argv[6] != '':
			t='Analysis of '+name_Metagenome+' VS 16S Microbial finished'
			m='Analysis of '+metagenome+' VS 16S Microbial finished in '+str(endTime(start_Time))+'\nNo 16S reads were founded.'
			send_mail(mail,t,m)
	
	
	showinfo('MG-Digger information', m)


def main(typeblast,metagenome,item,group,evalue,cpus,mail):
	''' Main Programm'''
	start_Time = time.time()
	extension_test(metagenome)
	extension_test(item) 
	print evalue
	
	metagenome = os.path.abspath(metagenome)
	name_Metagenome = os.path.basename(metagenome)
	
	item = os.path.abspath(item)
	name_Item = os.path.basename(item)
	

	# Creation of specific analysis folder -> metagenome/item.
	prefix_Metagenome = os.path.splitext(name_Metagenome)[0]
	prefix_Item = os.path.splitext(name_Item)[0]

	analysis_Folder = prefix_Metagenome+"_vs_"+prefix_Item+"_"+typeblast

	if os.path.exists(analysis_Folder):
		if askyesno('Information', "Folder exist!\nDo you want to delete it?"):
			shutil.rmtree(analysis_Folder)
			os.mkdir(analysis_Folder) 
		else:
			showwarning('Information', 'Update canceled!')

	else: 
		os.mkdir(analysis_Folder) 

	print "Start Analysis "+name_Metagenome+" and "+name_Item+"..."

	
	# Move to metagenome/subject folder
	work_Folder=os.getcwd()
	directory=work_Folder+"/"+analysis_Folder
	os.chdir(directory)


	print "5%"
	print "# Database formatting : "+name_Metagenome

	# Creation of a cvs file with the tblastn program.
	outputfile=prefix_Metagenome+"_VS_"+prefix_Item+"_"+typeblast+".csv"

	if typeblast == 'tblastn':
		# Creation of metagenome database file with makeblastdb program.
		dbmetagenome=prefix_Metagenome+".db"
		os.system("makeblastdb -in "+metagenome+" -dbtype nucl -out "+dbmetagenome)

		# Display on the screen.
		print "10%"
		print "# "+typeblast+" "+name_Item+" vs "+name_Metagenome+"..."

		# Run of tblastn program.
		os.system("tblastn -query "+item+" -db "+dbmetagenome+" -num_threads "+str(cpus)+" -evalue "+str(evalue)+" -max_target_seqs 1000000000 -soft_masking yes -outfmt \"6 qseqid sseqid bitscore score qcovs evalue pident length qlen slen qstart qend sstart send qseq sseq\" -out "+outputfile)

		
		
	elif typeblast == 'blastx':
		# Creation of metagenome database file with makeblastdb program.
		dbitem=prefix_Item+".db"
		os.system("makeblastdb -in "+item+" -dbtype prot -out "+dbitem)
		
		# Display on the screen.
		print "10%"
		print "# "+typeblast+" "+name_Item+" vs "+name_Metagenome+"..."

		# Run of blastx program.
		os.system("blastx -query "+metagenome+" -db "+dbitem+" -num_threads "+str(cpus)+" -evalue "+str(evalue)+" -max_target_seqs 10 -outfmt \"6 qseqid sseqid bitscore score qcovs evalue pident length qlen slen qstart qend sstart send qseq sseq\" -out "+outputfile)
		
	elif typeblast == 'blastp':
		# Creation of metagenome database file with makeblastdb program.
		dbitem=prefix_Item+".db"
		os.system("makeblastdb -in "+item+" -dbtype prot -out "+dbitem)

		# Display on the screen.
		print "10%"
		print "# "+typeblast+" "+name_Item+" vs "+name_Metagenome+"..."

		# Run of blastx program.
		os.system("blastp -query "+metagenome+" -db "+dbitem+" -num_threads "+str(cpus)+" -evalue "+str(evalue)+" -max_target_seqs 10 -outfmt \"6 qseqid sseqid bitscore score qcovs evalue pident length qlen slen qstart qend sstart send qseq sseq\" -out "+outputfile)



	elif typeblast== 'blastn':
		# Creation of metagenome database file with makeblastdb program.
		dbitem=prefix_Item+".db"
		os.system("makeblastdb -in "+item+" -dbtype nucl -out "+dbitem)

		# Display on the screen.
		print "10%"
		print "# "+typeblast+" "+name_Item+" vs "+name_Metagenome+"..."

		# Run of blastn program.
		os.system("blastn -query "+metagenome+" -db "+dbitem+" -num_threads "+str(cpus)+" -evalue "+str(evalue)+" -max_target_seqs 10 -outfmt \"6 qseqid sseqid bitscore score qcovs evalue pident length qlen slen qstart qend sstart send qseq sseq\" -out "+outputfile)
	
	try:	
		# Remove temporary database files	
		listeext=['*.nhr','*.nin','*.nsq','*.nal','*.phr','*.pin','*.psq','*.pal']
		for file1 in os.listdir(directory):
			for i in listeext:
				if fnmatch.fnmatch(file1,i):
					os.remove(file1)
	except:
		pass

	
	# Test if there is matching, we continue analysis otherwise tools are stopped 
	if os.path.isfile(outputfile) and os.path.getsize(outputfile) > 0 :
	
		# Display on the screen	
		print "30%"
		print "# Please wait, extraction of sequences from "+name_Metagenome+"..."

		# Script python to extract reads of metagenome matching with subject and count the number of total reads .
		cptreads = metagenome_sequences_match(metagenome,outputfile,typeblast)	


		# We take the fasta file in which subject vs metagenome  is the best match
		seqmatchfile = "seqmatch_for_"+outputfile+"_analyse.fasta"
	
		# Count the total sequences in metagenome
		cpt1 = 0
		for sequences in SeqIO.parse(seqmatchfile, "fasta"):
			cpt1 += 1
			
		
		if typeblast == 'blastn':
			typeblast2 ='blastn'
			dbdatabase = 'NT' 
		else:
			typeblast2 ='blastx'
			dbdatabase= 'NR'

		# Dispaly on the screen.
		print "50%"
		print "# Wait "+typeblast2+" "+seqmatchfile+" VS "+dbdatabase
		print "# Analysis of "+str(cpt1)+" reads..."
		print "# Estimation of finished time: "+str(estimate_time(typeblast,cpus, cpt1))
	
		if typeblast == 'blastn':
			# Creation of file containing result from blast program.
			seqdumpfile="result_blastn_for_"+prefix_Metagenome+"_vs_NT.csv"	
			# Blast of metagenome vs NR database. We can know if subject is the best match for every reads.		
			os.system ("blastn -query "+seqmatchfile+" -db \"16SMicrobial nt\" -num_threads "+str(cpus)+" -evalue "+str(evalue)+" -max_target_seqs 15 -outfmt \"6 qseqid sseqid stitle sskingdoms sscinames bitscore score qcovs evalue pident length qlen slen qstart qend sstart send qseq sseq\" -out "+seqdumpfile)
			

		
		elif typeblast == 'blastp':
			# Creation of file containing result from blast program.
			seqdumpfile="result_blastp_for_"+prefix_Metagenome+"_vs_NR.csv"
			# Blast of metagenome vs NR database. We can know if subject is the best match for every reads.
			os.system ("blastp -query "+seqmatchfile+" -db nr -num_threads "+str(cpus)+" -evalue "+str(evalue)+" -max_target_seqs 15 -outfmt \"6 qseqid sseqid stitle sskingdoms sscinames bitscore score qcovs evalue pident length qlen slen qstart qend sstart send qseq sseq\" -out "+seqdumpfile)
			

		else :
			# Creation of file containing result from blast program.
			seqdumpfile="result_blastx_for_"+prefix_Metagenome+"_vs_NR.csv"
			# Blast of metagenome vs NR database. We can know if subject is the best match for every reads.
			os.system ("blastx -query "+seqmatchfile+" -db nr -num_threads "+str(cpus)+" -evalue "+str(evalue)+" -max_target_seqs 15 -outfmt \"6 qseqid sseqid stitle sskingdoms sscinames bitscore score qcovs evalue pident length qlen slen qstart qend sstart send qseq sseq\" -out "+seqdumpfile)

	
		# If subject belongs to the top of the best hits for a read, we can keep it for subsequent analysis by user.
		if  os.path.getsize(seqdumpfile) > 0 and os.path.isfile(seqdumpfile):
		
			# Dispaly on the screen.print "75%"
			print "# Sort of reads to analysis"		
		
			if os.path.getsize(seqdumpfile) > 0 and os.path.isfile(seqdumpfile):  
			
				print "# Extraction of one hsp per read..."
			
				# Extract one hsp per reads.
				one_hsp(typeblast,seqdumpfile,group)
			
			
				resultonehsp="result_one_hsp_per_read.csv"
				if  os.path.getsize(resultonehsp) > 0 and os.path.isfile(resultonehsp):
				
					# Display on the screen.				
					print "# Creation of final reads fasta file..."
				
					# Extract reads in final fasta file.				
					extract_per_items(metagenome,seqmatchfile,resultonehsp)
				
					# Count the numbers of final reads.
					cpt2=0
					with open(resultonehsp) as f:
	    					cpt2=sum(1 for _ in f)
					

					#Display on the screen		
					print "85%"
					print "# Get lineage for taxonomy studies for "+str(cpt2)+" species..."
					
									
					# Get  stats file
					get_stats_species(resultonehsp)
					
					# Name of statistic files.
					numberitemstmp="stats_tmp.csv"
					statsfinal="Stats_"+name_Metagenome+"_VS_"+name_Item+"_blastn.csv"

					# Get lineage of species
					taxonomy(numberitemstmp,statsfinal)
				
					# Sort colomn lineage				
					sort_csv(statsfinal,4)
				

					# Add explicit line in read files
					with open(statsfinal, 'r+') as file:
						text = file.read()
	       					file.seek(0)
						file.write("# Metagenome: "+name_Metagenome+";\n# Subjects: "+name_Item+";\n# E-value: "+str(evalue)+";\n# Reads total: "+str(cptreads)+";\n# Reads extracted: "+str(cpt1)+";\n# Reads found: "+str(cpt2)+";\n# Analysis time: "+str(endTime(start_Time))+";\n# Number of CPUs used: "+str(cpus)+";\n#\n# Protein ORFs\n# Numbers\tSubject name\tTaxonomy ID\tLineage\n"+text)
					with open(resultonehsp, 'r+') as file:
						text = file.read()
	       					file.seek(0)
						file.write("Query Seq-id\tSubject Seq-id\tTitle\tGroup\tScientific name(s)\tMax score\tTotal score\tQuery coverage\tE-value\tPercentage of identical matches\tAlignment length (protein)\tQuery sequence length (nucleotide)\tSubject sequence length (protein)\tStart of alignment in query (nucleotide)\tEnd of alignment in query (nucleotide)\tStart of alignment in subject (protein)\tEnd of alignment in subject (protein)\tAligned part of query sequence\tAligned part of subject sequence\n"+text)
				

					# Display on the screen
					print "100%"
					print "# Analysis metagenome "+name_Metagenome+" with "+name_Item+" end!"
					print str(cpt2)+" reads found in "+str(endTime(start_Time))
				
					# Remove temporary files
					os.remove(numberitemstmp)
					os.remove(seqmatchfile)
					os.remove(outputfile) 
		
					# Send mail
					if sys.argv[6] != '':
						t='Analysis of '+name_Metagenome+' VS '+name_Item+' finished'
						m='Analysis of '+name_Metagenome+' VS '+name_Item+' finished in '+str(endTime(start_Time))+'\n'+str(cpt2)+' reads were founded.\nPlease verify the folder : '+directory
						send_mail(mail,t,m,statsfinal)
	

				else :
					#Delete empty files
					del_folder(directory)

					# Add satistic files
					with open("Stats_"+name_Metagenome+"_VS_"+name_Item+"_tblastn.txt", "w") as file:	
						file.write("# Metagenome: "+name_Metagenome+";\n# Subjects: "+name_Item+";\n# E-value: "+str(evalue)+";\n# Reads extracted: "+str(cpt1)+"\n# Analysis time: "+str(endTime(start_Time))+";\n#\n#\nNo match between "+prefix_Metagenome+" and "+prefix_Item+" !")
							
					# Display on the screen	
					print "100%"
					print "# No "+prefix_Item+" in "+prefix_Metagenome+"!"
					print "Analysis finished in "+str(endTime(start_Time))

					# Send mail
					if sys.argv[6] != '':
						t='Analysis of '+name_Metagenome+' VS '+name_Item+' finished'
						m='Analysis of '+metagenome+' VS '+item+' finished in '+str(endTime(start_Time))+'\nNo reads were founded.'
						send_mail(mail,t,m)
			
		
			else :
				#Delete empty files
				del_folder(directory)

				# Add satistic files	
				with open("Stats_"+name_Metagenome+"_VS_"+name_Item+"_tblastn.txt", "w") as file:
					file.write("# Metagenome: "+name_Metagenome+";\n# Subjects: "+name_Item+";\n# E-value: "+str(evalue)+";\n# Reads total: "+str(cptreads)+";\n# Analysis time: "+str(endTime(start_Time))+";\n#\n#\nNo match between "+name_Metagenome+" and "+name_Item+" !")

				# Display on the screen	
				print "100%"
				print "# No "+prefix_Item+" in "+prefix_Metagenome+"!"
				print "Analysis finished in "+str(endTime(start_Time))
			
				# Send mail
				if sys.argv[6] != '':
					t='Analysis of '+name_Metagenome+' VS '+name_Item+' finished'
					m='Analysis of '+metagenome+' VS '+item+' finished in '+str(endTime(start_Time))+'\nNo reads were founded.'
					send_mail(mail,t,m)
		
		else :
			#Delete empty files
			del_folder(directory)
			
			# Add satistic files	
			with open("Stats_"+name_Metagenome+"_VS_"+name_Item+"_tblastn.txt", "w") as file:
				file.write("# Metagenome: "+name_Metagenome+";\n# Subjects: "+name_Item+";\n# E-value: "+str(evalue)+";\n# Reads total: "+str(cptreads)+";\n# Analysis time: "+str(endTime(start_Time))+";\n#\n#\nNo match between "+name_Metagenome+" and "+name_Item+" !")

			# Display on the screen	
			print "100%"
			print "# No "+name_Item+" in "+name_Metagenome+"!"
			print "Analysis finished in "+str(endTime(start_Time))

			# Send mail
			if sys.argv[6] != '':
				t='Analysis of '+name_Metagenome+' VS '+name_Item+' finished'
				m='Analysis of '+metagenome+' VS '+item+' finished in '+str(endTime(start_Time))+'\nNo reads were founded.'
				send_mail(mail,t,m)



		

	# If no matching beetween Metagenome and subject
	else :
		# Count of number read of metagenome
		cptreads = 0
		for sequences in SeqIO.parse(metagenome, "fasta"):
			cptreads += 1

		#Delete empty files
		del_folder(directory)	
		
		# Add satistic files	
		with open("Stats_"+name_Metagenome+"_VS_"+name_Item+"_tblastn.txt", "w") as file:
			file.write("# Metagenome: "+name_Metagenome+";\n# Subjects: "+name_Item+";\n# E-value: "+str(evalue)+";\n# Reads total: "+str(cptreads)+";\n# Analysis time: "+str(endTime(start_Time))+";\n#\n#\nNo match between "+prefix_Metagenome+" and "+prefix_Item+" !")

		# Display on the screen	
		print "100%"
		print "# No match between "+prefix_Metagenome+" and "+prefix_Item+"!"
		print "Analysis finished in "+str(endTime(start_Time))

		# Send mail
		if sys.argv[6] != '':
			t='Analysis of '+name_Metagenome+' VS '+name_Item+' finished'
			m='Analysis of '+metagenome+' VS '+item+' finished in '+str(endTime(start_Time))+'\nNo reads were founded.'
			send_mail(mail,t,m)
	

	showinfo('MG-Digger information', m)


######################## Main programm ################################################################################################
if __name__ == "__main__":

	#folder = str(os.path.dirname(os.path.abspath(__file__)))
	folder_s = str(os.path.realpath(os.path.abspath(os.path.split(inspect.getfile( inspect.currentframe() ))[0])))

	if NCBI_Folder== '':
		NCBI_Folder= folder_s+'/NCBI_tools'
		sra_prog_folder= folder_s+'/NCBI_tools/sratoolkit/bin'

	#NCBI databases
	Blast_Folder=NCBI_Folder+"/blast/db/"
	TaxID_Folder=NCBI_Folder+"/pub/taxonomy"
	
	Archeae_list=NCBI_Folder+"/Archeae_list.txt"
	Bacteria_list=NCBI_Folder+"/Bacteria_list.txt"
	Eukaryota_list=NCBI_Folder+"/Eukaryota_list.txt"
	Megavirales_list=NCBI_Folder+"/Megavirales_list.txt"
	Viruses_list=NCBI_Folder+"/Viruses_list.txt"
	names_file=TaxID_Folder+"/names.dmp"
	nodes_file=TaxID_Folder+"/nodes.dmp"


	## Construction of window tool
	root = Tk()
	root.wm_title("MG-Digger")
	root.resizable(0,0)
	


	## Menu bar
	menubar = Menu(root)

	menu1 = Menu(menubar, tearoff=0)
	menu1.add_command(label="Exit", command=root.quit)
	menubar.add_cascade(label="File", menu=menu1)

	menu2 = Menu(menubar, tearoff=0)
	menu2.add_command(label="Set path", command=setpath)
	menu2.add_command(label="Update database", command=update)
	menubar.add_cascade(label="Edition", menu=menu2)

	menu3 = Menu(menubar, tearoff=0)
	menu3.add_command(label="Cleaning of metagenome", command=cleaner)
	menubar.add_cascade(label="Option", menu=menu3)

	menu4 = Menu(menubar, tearoff=0)
	menu4.add_command(label="About...", command=about)
	menubar.add_cascade(label="Help", menu=menu4)

	root.config(menu=menubar)
	
	
	# Logo
	photo = PhotoImage(file=folder_s+"/"+'mi.gif')
	canvas = Canvas(root,width=100, height=100)
	canvas.create_image(0, 0, anchor=NW, image=photo)
	canvas.pack()
		
	# Tool title
	label_title = Label(root, text="MG-Digger v1.0",bg="green").pack()


	# Choose files
	label_files = LabelFrame(root, text="Choose the files:", borderwidth=5 ,padx=20, pady=20)
	label_files.pack(fill="both", expand="yes")

	# Choose metagenome
	label_metagenome = Frame(label_files, borderwidth=0)
	label_metagenome.pack(fill="both", expand="yes")

	Button(label_metagenome, text ="Select Metagenome", command=select_metagenome).pack(side=LEFT, padx=5, pady=5)
	text_Metagenome = StringVar() 
	LabelResultat = Label(label_metagenome, textvariable = text_Metagenome, fg ='red', bg ='white')
	LabelResultat.pack(side = LEFT, padx = 5, pady = 5)

	# Choose item
	label_item = Frame(label_files, borderwidth=0)
	label_item.pack(fill="both", expand="yes")

	Button(label_item, text ="Select reference fasta file", command=select_item).pack(side=LEFT, padx=5, pady=5)
	text_Item = StringVar() 
	LabelResultat2 = Label(label_item, textvariable = text_Item, fg ='red', bg ='white')
	LabelResultat2.pack(side = LEFT, padx = 5, pady = 5)

	# Option window.
	label_option = LabelFrame(root, text="Option:", borderwidth=5, padx=20, pady=20)
	label_option.pack(fill="both", expand="yes")

	label_option1 = LabelFrame(label_option, borderwidth=0, padx=10, pady=10)
	label_option1.pack(side = LEFT)

	#Group
	label_group = LabelFrame(label_option1, text="Define a group of interest", borderwidth=2, height=5, relief=GROOVE)
	label_group.pack(side=TOP) 

	listbox1 = Listbox(label_group)

		
	listbox1.insert(1, "All")	
	listbox1.insert(2, "Archeae")
	listbox1.insert(3, "Bacteria")
	listbox1.insert(4, "Eukaryota")
	listbox1.insert(5, "Megavirales")	
	listbox1.insert(6, "Viruses")

	listbox1.pack()

	listbox1.bind()


	# Option extraction
	label_extraction = LabelFrame(label_option1, text="Select BLAST method:")
	label_extraction.pack(side=BOTTOM)
	 
	typeblast = StringVar() 
	bouton1 = Radiobutton(label_extraction, text="Protein (tblastn)", variable=typeblast, value="tblastn")
	bouton2 = Radiobutton(label_extraction, text="Protein (blastx) (Slow/More accurately)", variable=typeblast, value="blastx")
	bouton3 = Radiobutton(label_extraction, text="Protein (blastp) (Protein Metagenome)", variable=typeblast, value="blastp")	
	bouton4 = Radiobutton(label_extraction, text="Nucleotide (blastn)", variable=typeblast, value="blastn")
	bouton5 = Radiobutton(label_extraction, text="16S Analysis", variable=typeblast, value="16S")
	bouton1.pack()
	bouton2.pack()
	bouton3.pack()
	bouton4.pack()
	bouton5.pack()
	bouton1.select()



	label_option2 = LabelFrame(label_option, borderwidth=0, padx=10, pady=10)
	label_option2.pack(side=RIGHT)

	# E-value
	label_evalue = LabelFrame(label_option2, text="Enter an e-value cutoff", borderwidth=2,relief=GROOVE)
	label_evalue.pack(side=TOP)

	evalue = StringVar() 
	evalue.set("1e-3")
	entree = Entry(label_evalue, textvariable=evalue, width=30, highlightthickness= 40)
	entree.pack()

	# E-mail
	label_mail = LabelFrame(label_option2, text="Enter an e-mail (optional)", borderwidth=2, relief=GROOVE)
	label_mail.pack()

	mail = StringVar()
	mail.set("")
	entree = Entry(label_mail, textvariable=mail, width=30, highlightthickness= 40)
	entree.pack()

	#CPU
	label_cpus = LabelFrame(label_option2, text="Number of cpus:", borderwidth=2, relief=GROOVE)
	label_cpus.pack()
	cpus= StringVar()
	s = Spinbox(label_cpus, from_=1, to=192, textvariable=cpus )
	s.pack()

	# Buttons
	Button(root, text ='Run', command=run).pack(side=LEFT, padx=5, pady=5)
	Button(root, text ='Cancel', command=root.quit).pack(side=RIGHT, padx=5, pady=5)

	root.mainloop()	
	
